export type NumType = number | bigint;
export declare const isNumType: (x: any) => x is NumType;
export type BytesType = Uint8Array;
export declare const isBytesType: (x: any) => x is Uint8Array;
export type PrimitiveType = NumType | BytesType;
export declare const isPrimitiveType: (x: any) => x is PrimitiveType;
export type BinaryLiterals = "int" | "uint" | "bytes" | "array" | "object" | "switch";
export type Endianness = "little" | "big";
export type NumberSize = 1 | 2 | 3 | 4 | 5 | 6;
export declare const numberMaxSize = 6;
export type NumSizeToPrimitive<Size extends number> = Size extends NumberSize ? number : bigint;
export type FixedConversion<FromType extends PrimitiveType, ToType> = {
    readonly to: ToType;
    readonly from: FromType;
};
export type CustomConversion<FromType extends PrimitiveType, ToType> = {
    readonly to: (val: FromType) => ToType;
    readonly from: (val: ToType) => FromType;
};
interface LayoutItemBase<BL extends BinaryLiterals> {
    readonly binary: BL;
}
interface PrimitiveFixedCustom<T extends PrimitiveType> {
    custom: T;
    omit?: boolean;
}
interface OptionalToFromCustom<T extends PrimitiveType> {
    custom?: FixedConversion<T, any> | CustomConversion<T, any>;
}
interface NumLayoutItemBase<T extends NumType, Signed extends Boolean> extends LayoutItemBase<Signed extends true ? "int" : "uint"> {
    size: T extends bigint ? number : NumberSize;
    endianness?: Endianness;
}
interface PrimitiveFixedNumLayoutItem<T extends NumType, Signed extends Boolean> extends NumLayoutItemBase<T, Signed>, PrimitiveFixedCustom<T> {
}
interface OptionalToFromNumLayoutItem<T extends NumType, Signed extends Boolean> extends NumLayoutItemBase<T, Signed>, OptionalToFromCustom<T> {
}
export interface FixedPrimitiveBytesLayoutItem extends LayoutItemBase<"bytes">, PrimitiveFixedCustom<BytesType> {
}
export interface FixedValueBytesLayoutItem extends LayoutItemBase<"bytes"> {
    readonly custom: FixedConversion<BytesType, any>;
}
export interface FixedSizeBytesLayoutItem extends LayoutItemBase<"bytes"> {
    readonly size: number;
    readonly custom?: CustomConversion<BytesType, any>;
}
export interface LengthPrefixedBytesLayoutItem extends LayoutItemBase<"bytes"> {
    readonly lengthSize?: NumberSize;
    readonly lengthEndianness?: Endianness;
    readonly custom?: CustomConversion<BytesType, any>;
}
export interface ArrayLayoutItem extends LayoutItemBase<"array"> {
    readonly lengthSize?: NumberSize;
    readonly lengthEndianness?: Endianness;
    readonly arrayItem: LayoutItem;
}
export interface ObjectLayoutItem extends LayoutItemBase<"object"> {
    readonly layout: Layout;
}
type PlainId = number;
type ConversionId = readonly [number, unknown];
type IdLayoutPair<Id extends PlainId | ConversionId, L extends Layout = Layout> = readonly [Id, L];
export interface SwitchLayoutItem extends LayoutItemBase<"switch"> {
    readonly idSize: NumberSize;
    readonly idTag?: string;
    readonly idEndianness?: Endianness;
    readonly idLayoutPairs: readonly IdLayoutPair<PlainId>[] | readonly IdLayoutPair<ConversionId>[];
}
type NumLayoutItem<Signed extends boolean = boolean> = PrimitiveFixedNumLayoutItem<number, Signed> | OptionalToFromNumLayoutItem<number, Signed> | PrimitiveFixedNumLayoutItem<bigint, Signed> | OptionalToFromNumLayoutItem<bigint, Signed>;
export type IntLayoutItem = NumLayoutItem<true>;
export type UintLayoutItem = NumLayoutItem<false>;
export type BytesLayoutItem = FixedPrimitiveBytesLayoutItem | FixedValueBytesLayoutItem | FixedSizeBytesLayoutItem | LengthPrefixedBytesLayoutItem;
export type LayoutItem = IntLayoutItem | UintLayoutItem | BytesLayoutItem | ArrayLayoutItem | ObjectLayoutItem | SwitchLayoutItem;
export type NamedLayoutItem = LayoutItem & {
    readonly name: string;
};
export type Layout = readonly NamedLayoutItem[];
type NameOrOmitted<T extends {
    name: string;
}> = T extends {
    omit: true;
} ? never : T["name"];
export type LayoutToType<L extends Layout> = {
    readonly [I in L[number] as NameOrOmitted<I>]: LayoutItemToType<I>;
};
type MaybeConvert<Id extends PlainId | ConversionId> = Id extends readonly [number, infer Converted] ? Converted : Id;
type IdLayoutPairArray = readonly IdLayoutPair<number>[] | readonly IdLayoutPair<ConversionId>[];
type IdLayoutPairsToTypeUnion<T extends IdLayoutPairArray, IdTag extends string> = T extends readonly [infer Head, ...infer Tail extends IdLayoutPairArray] ? Head extends IdLayoutPair<infer MaybeConversionId, infer L extends Layout> ? MaybeConvert<MaybeConversionId> extends infer Id ? LayoutToType<L> extends infer LT extends object ? {
    readonly [K in IdTag | keyof LT]: K extends keyof LT ? LT[K] : Id;
} | IdLayoutPairsToTypeUnion<Tail, IdTag> : never : never : never : never;
export type LayoutItemToType<I extends LayoutItem> = [
    I
] extends [NumLayoutItem] ? I["custom"] extends NumType ? I["custom"] : I["custom"] extends CustomConversion<infer FromType extends NumType, infer ToType> ? ToType : I["custom"] extends FixedConversion<infer FromType extends NumType, infer ToType> ? ToType : NumSizeToPrimitive<I["size"]> : [I] extends [BytesLayoutItem] ? I["custom"] extends CustomConversion<BytesType, infer ToType> ? ToType : I["custom"] extends FixedConversion<BytesType, infer ToType> ? ToType : BytesType : [I] extends [ArrayLayoutItem] ? readonly LayoutItemToType<I["arrayItem"]>[] : [I] extends [ObjectLayoutItem] ? LayoutToType<I["layout"]> : [I] extends [SwitchLayoutItem] ? IdLayoutPairsToTypeUnion<I["idLayoutPairs"], I["idTag"] extends string ? I["idTag"] : "id"> : never;
export {};
//# sourceMappingURL=layout.d.ts.map